package org.openxma.dsl.platform.assembler;

/**
 * This interface provides access to so called DtoAssemblers. DtoAssemblers
 * implement a specific interface {@link DtoAssembler} in order to provide their
 * dto assembling and disassembling functionality
 */
public interface DtoAssemblerRegistry {

    /**
     * @param <T>
     *            the class of the DTO
     * @param dtoClass
     *            the dto class to get the {@link DtoAssembler} for
     * @return a {@link DtoAssembler} which could expose <b>default</b> or
     *         <b>custom</b> behavior.<br>
     *         The kind of behavior (default or custom) returned depends on
     *         whether<br>
     *         a custom implementation of a {@link DtoAssembler} exists for a
     *         given dto.
     */
    <T> DtoAssembler<T> getDtoAssembler(Class<T> dtoClass);

}
