package org.openxma.dsl.platform.enums;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import at.spardat.xma.datasource.ProviderResultServer;
import at.spardat.xma.datasource.RessourceBundleProviderServer;
import at.spardat.xma.datasource.TableSpec;
import at.spardat.xma.datasource.TabularDataRow;

public class I18nEnums {

	private final static Map<String,TabularDataRow> EMPTY_TABLE_CACHE = new HashMap<String,TabularDataRow>();

	private static Map<String,Map<String,TabularDataRow>> cache = new HashMap<String,Map<String,TabularDataRow>>();

	static <A extends Enum<?>> Map<String,TabularDataRow> getTableCache(Class<?> enumClass, Locale l) {
		String bundleName = enumClass.getName();
		Map<String,TabularDataRow> tableCache = cache.get(bundleName);
		if (tableCache == null) {
			try {
				ResourceBundle.getBundle(bundleName,l);
			} catch (Exception ex) {
				 cache.put(bundleName,EMPTY_TABLE_CACHE);
				 return EMPTY_TABLE_CACHE;
			}
	        RessourceBundleProviderServer ps = new RessourceBundleProviderServer();
	        TableSpec spec = new TableSpec();
	        spec.setProperty("bundle",enumClass.getName());
	        spec.setLocale(l);
	        ProviderResultServer result = ps.provideTable(null, spec, 0);
	        tableCache = new HashMap<String,TabularDataRow>();
	        for (int i=0;i<result.table_.size();i++) {
	        	TabularDataRow row = result.table_.getRow(i);
	        	tableCache.put(row.get(0).toString(), row);
	        }
	        cache.put(enumClass.getName(),tableCache);
		}
		return tableCache;
	}

	static TabularDataRow getTabularDataRowFor(Enum<?> value, Locale l) {
		Map<String,TabularDataRow> tableCache = getTableCache(value.getClass(),l);
		if (tableCache == EMPTY_TABLE_CACHE) return null;
		return tableCache.get(keyOf(value));
	}

	public static <A extends Enum<?>> String shortValueOf(A value, Locale l) {
		if (l == null) l=Locale.getDefault();
		if (value instanceof I18nEnum) {
			I18nEnum i18Enum = (I18nEnum)value;
			return i18Enum.getShortValue(l);
		} else if (value instanceof Enum<?>) {
			String shortValue = shortValueFromProvider(value,l);
			if (shortValue != null) return shortValue;
		}
		return value.toString();
	}

	public static <A extends Enum<?>> String longValueOf(A value, Locale l) {
		if (l == null) l=Locale.getDefault();
		if (value instanceof I18nEnum) {
			I18nEnum i18Enum = (I18nEnum)value;
			return i18Enum.getLongValue(l);
		} else if (value instanceof Enum<?>) {
			String longValue = longValueFromProvider(value,l);
			if (longValue != null) return longValue;
		}
		return value.toString();
	}

	public static <A extends Enum<?>> String shortValueFromProvider(A value, Locale l) {
		if (l == null) l=Locale.getDefault();
		TabularDataRow row = getTabularDataRowFor(value,l);
		if (row != null) {
			return row.get(1).toString();
		}
		return null;
	}

	public static <A extends Enum<?>> String longValueFromProvider(A value, Locale l) {
		if (l == null) l=Locale.getDefault();
		TabularDataRow row = getTabularDataRowFor(value,l);
		if (row != null) {
			return row.get(2).toString();
		}
		return null;
	}

	private static <A extends Enum<?>> A[] getEnumValues(Class<A> clazz) {
		try {
			Method valuesMethod = clazz.getMethod("values");
			return (A[])valuesMethod.invoke(clazz);
		} catch (Exception e) {
			throw new RuntimeException("Can not get enum values for '"+clazz.getName()+"'",e);
		}
	}

    public static <A extends Enum<?>> String keyOf(A value) {
    	if (value == null) return null;
    	if (value instanceof I18nEnum) {
			I18nEnum i18Enum = (I18nEnum)value;
			return i18Enum.getKey();
		}
		return value.toString();
    }

	public static<A extends Enum<?>> A valueOfKey(Class<A> clazz, String key) {
		return valueOfKey(getEnumValues(clazz),key);
	}

    public static <A extends Enum<?>> A valueOfKey(A[] values, String code) {
    	if (code == null || code.length() == 0) return null;
    	for (A item: values) {
    		String key = keyOf(item);
    		if (key != null && key.equals(code)) return item;
    	}
    	throw new RuntimeException("No enum const for key '" + code + "'");
    }

}

